@@ -30,7 +30,7 @@
 	    define-ffi-module
 	    compile-ffi-file
 	    load-include-file
-	    ccode->scheme)
+	    ccode->sexp)
   #:use-module (ice-9 format)
   #:use-module (ice-9 match)
   #:use-module (ice-9 popen)
@@ -61,7 +61,7 @@
   #:use-module ((nyacc lex) #:select (cnumstr->scm))
   #:use-module ((nyacc util) #:select (ugly-print))
   #:re-export (*nyacc-version*)
-  #:version (2 00 0))
+  #:version (2 00 1))
 
 ;; maybe change to a record-type
 (define *options* (make-parameter '()))
@@ -622,7 +622,7 @@
     (match (md-tail mdecl)
       (`((enum-def . ,_)) (list (sfsym "wrap-~a" name) mname))
       (`((enum-ref . ,_)) (list (sfsym "wrap-~a" name) mname))
-      (__ #f))))
+      (__ `(make-cdata ,(string->symbol name) ,mname)))))
 
 (define (wrap-mdecl mdecl)
   (let ((defined (*defined*))
@@ -1396,6 +1396,7 @@
 	  (process-decls ffi-decls udecls ;; udict <= swap failed 01 Dec 2018
 			 (append def-defined ext-defd) #:declf declf))
       (lambda (defined seed)
+        (sferr "ffi-help ended with ~s defined objects\n" (length defined))
 	;; Set ffimod-defined for including, but removed built-in types.
 	(let* ((bity (car def-defined))	; first built-in type
 	       (defd (let loop ((res '()) (defs defined))
@@ -1421,16 +1422,26 @@
 
 ;; === translators ================
 
-;; @deffn {Procedure} ccode->scheme string => tree
+;; @deffn {Procedure} ccode->sexp string [attrs] => tree
 ;; Convert a snippet of C code to list of scheme forms.
 ;; @example
-;; 
+;; > (ccode->sexp "double sqrt(doubld);")
+;; $1 = 
 ;; @end example
 ;; @end deffn
-(define* (ccode->scheme code-string #:key inc-dirs cpp-defs)
-  ;; need gensym on load-foreign-function-search
-  #f)
-
+(define* (ccode->sexp code #:optional attrs)
+  (let* ((tree (parse-code code attrs))
+	 (udict (c99-trans-unit->udict/deep tree))
+	 (udecls (c99-trans-unit->udict tree))
+	 (ffi-decls (map car udecls))
+         (cnvt (lambda (udent defined seed)
+                 (cnvt-udecl (cdr udent) udict defined seed))))
+    (*udict* udict)
+    (*ddict* (udict-enums->ddict udict (*ddict*)))
+    `(begin
+       ,@(call-with-values
+             (lambda () (fold-values cnvt udecls '() '()))
+           (lambda (defined forms) (reverse forms))))))
 
 ;; @deffn {Procedure} load-include-file filename [#pkg-config pkg]
 ;; This is the functionality that Ludo was asking for: to be at guile
